Screen {
    background: $panel;
}

Input {
    dock: top;    
    width: 100%;
    height: 1;
    padding: 0 1;
    margin: 1 1 0 1;    
}

#results {
    width: auto;   
    min-height: 100%;
}

#results-container {
    background: $background 50%;
    overflow: auto;    
    margin: 1 2;    
    height: 100%;
}
