Screen {
    layers: log ball;
}

TextLog {
    layer: log;
}

PlayArea {
    opacity: 0%;
    layer: ball;

}
Ball {
    layer: ball;
    width: auto;
    height: 1;
    background: $secondary;
    border: tall $secondary;
    color: $background;
    box-sizing: content-box;
    text-style: bold;
    padding: 0 4;
}
