Screen {
    background: white;
    color: black;
    layout: horizontal;
}
Label {
    width: 20;
    height: 10;
    content-align: center middle;
}

.paul {
    offset: 8 2;
    background: red 20%;
    border: outer red;
    color: red;
}

.duncan {
    offset: 4 10;
    background: green 20%;
    border: outer green;
    color: green;
}

.chani {
    offset: 0 -3;
    background: blue 20%;
    border: outer blue;
    color: blue;
}
